/*
	"Crotchet"; a tool for plucking

		Copyright (C) 2017  Kassen Oud
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


		Configuration;
	It's recommended to keep the jumper on the DAC submodule when running this code.
	This way your signal should stay in the -5V to +5V range.

		Manual;
	Simple 'Karplus Strong' physical model of a string.

	Knob 1/ Mod 1; Pitch. Keep Knob 1 fully open for V/Oct tracking.
	Knob 2/ Mod 2; Frequency damping, higher values result in a brighter tone that
		sounds longer.
	Tr/G / Button A; 'Pluck' the string. Once this signal goes low or the button
		is released, the tone will damp more quickly.
	Button B; Toggle "bowed" or "blown" mode, which will keep adding noise while
		the trigger is high. This leads to a far more sustained sound.
	Led A; indication for the Tr/G input.
	Led B; indicator for the "bowed" mode.


	Philosophy
		(in case you'd like to know what's going on) 
		the "Karplus Strong" method of creating plucked-string-like sounds
		is quite old and quite simple. It involves an array of values that get
		read in a loop and output to the DAC. TO 'pluck' our imaginary string
		the array is filled with random values so all notes start as pure noise.
		From there on, as we read from this array we also average between the
		value at the current location and the last value we calculated. The result
		of that calculation is what we listen to, but we also write it back to 
		that position in the array. This averaging, or smoothing, gradually turns
		the periodic noise to a sine as the sound decays. This sine will have a
		period equal to the length of time it takes us to go this array once
		(double that if we invert the signal in the process). To put it in other
		words; the frequency of the output will be the samplerate divided by the
		number of values in our array.

		Now the question becomes; "How do we tune this?" We could vary the length
		of the array, but then we end up with less frequency resolution and it's
		not so clear how we would deal with pitch modulation of sounding notes.
		We could also interpolate, but that's far more complicated and we lose the
		charm of the simple algorithm. Most of the time those are our main options
		in the digital domain as the samplerate will be fixed by the soundcard. On
		the Tool, however, we can simply modulate the samplerate itself, saving a
		lot of work and preventing artefacts on modulation. Well, artefacts not
		caused by the lack of an anti-aliasing filter...

		There remains the issue that with this algorithm higher notes decay far
		more quickly than lower ones. To some degree that's also what happens in
		nature, and moreover; that's simply an inherent property of this method of
		synthesis. At least there is the option to keep adding noise to the system,
		so you can make high-pitched notes with a long duration, those just won't
		sound much like a plucked string.
*/
 
#include "SPI.h" // spi library
#include <avr/interrupt.h> //library for using interupts
#include <avr/pgmspace.h>
#include "kas-button.h" //button debouncing

static byte mod_a_pin = A1;
static byte mod_b_pin = A2;
static byte trigger_pin = 4;
static byte but_A_pin = 2;
static byte but_B_pin = 9;
static byte led_a = 3;
static byte led_b = 5;

int timer1_pre_load = 1;
byte phase = 0;
bool sustain = false;
bool bowed = false;


Button trigger;
Button but_A;
Button but_B;

uint16_t output;
uint16_t last_out;

byte mix_A = 15;
byte mix_B = 1;

//"xorshift" noise generator because "rand()" costs quite a bit of CPU
//borrowed from http://www.arklyffe.com/main/2010/08/29/xorshift-pseudorandom-number-generator/
uint16_t set_seed = 1;
uint16_t rand_seed = 1;
uint16_t rng(void)
{
	rand_seed ^= (rand_seed << 13);
	rand_seed ^= (rand_seed >> 9);
	return rand_seed ^= (rand_seed << 7);
}

//exponential table for pre-loading timer1
const uint16_t freqTable[] PROGMEM = {
57890, 57916, 57942, 57968, 57993, 58019, 58044, 58069, 58095, 58120, 58145, 58170, 58195, 58220, 58244,  
58269, 58293, 58318, 58342, 58367, 58391, 58415, 58439, 58463, 58487, 58511, 58534, 58558, 58582, 58605,  
58629, 58652, 58675, 58698, 58721, 58744, 58767, 58790, 58813, 58836, 58858, 58881, 58903, 58926, 58948,  
58970, 58993, 59015, 59037, 59059, 59081, 59102, 59124, 59146, 59167, 59189, 59210, 59232, 59253, 59274,  
59295, 59316, 59337, 59358, 59379, 59400, 59421, 59441, 59462, 59483, 59503, 59523, 59544, 59564, 59584,  
59604, 59624, 59644, 59664, 59684, 59704, 59723, 59743, 59763, 59782, 59802, 59821, 59840, 59860, 59879,  
59898, 59917, 59936, 59955, 59974, 59992, 60011, 60030, 60048, 60067, 60085, 60104, 60122, 60140, 60159,  
60177, 60195, 60213, 60231, 60249, 60267, 60285, 60302, 60320, 60338, 60355, 60373, 60390, 60408, 60425,  
60442, 60459, 60476, 60494, 60511, 60528, 60544, 60561, 60578, 60595, 60612, 60628, 60645, 60661, 60678,  
60694, 60711, 60727, 60743, 60759, 60775, 60792, 60808, 60824, 60839, 60855, 60871, 60887, 60903, 60918,  
60934, 60949, 60965, 60980, 60996, 61011, 61026, 61042, 61057, 61072, 61087, 61102, 61117, 61132, 61147,  
61162, 61176, 61191, 61206, 61220, 61235, 61249, 61264, 61278, 61293, 61307, 61321, 61336, 61350, 61364,  
61378, 61392, 61406, 61420, 61434, 61448, 61462, 61475, 61489, 61503, 61516, 61530, 61544, 61557, 61570,  
61584, 61597, 61610, 61624, 61637, 61650, 61663, 61676, 61689, 61702, 61715, 61728, 61741, 61754, 61767,  
61779, 61792, 61805, 61817, 61830, 61842, 61855, 61867, 61880, 61892, 61904, 61917, 61929, 61941, 61953,  
61965, 61977, 61989, 62001, 62013, 62025, 62037, 62049, 62061, 62072, 62084, 62096, 62107, 62119, 62131,  
62142, 62153, 62165, 62176, 62188, 62199, 62210, 62221, 62233, 62244, 62255, 62266, 62277, 62288, 62299,  
62310, 62321, 62332, 62343, 62353, 62364, 62375, 62385, 62396, 62407, 62417, 62428, 62438, 62449, 62459,  
62470, 62480, 62490, 62501, 62511, 62521, 62531, 62541, 62552, 62562, 62572, 62582, 62592, 62602, 62611,  
62621, 62631, 62641, 62651, 62661, 62670, 62680, 62690, 62699, 62709, 62718, 62728, 62737, 62747, 62756,  
62766, 62775, 62784, 62794, 62803, 62812, 62821, 62830, 62840, 62849, 62858, 62867, 62876, 62885, 62894,  
62903, 62912, 62920, 62929, 62938, 62947, 62956, 62964, 62973, 62982, 62990, 62999, 63007, 63016, 63024,  
63033, 63041, 63050, 63058, 63067, 63075, 63083, 63092, 63100, 63108, 63116, 63124, 63133, 63141, 63149,  
63157, 63165, 63173, 63181, 63189, 63197, 63205, 63213, 63220, 63228, 63236, 63244, 63251, 63259, 63267,  
63275, 63282, 63290, 63297, 63305, 63312, 63320, 63327, 63335, 63342, 63350, 63357, 63365, 63372, 63379,  
63386, 63394, 63401, 63408, 63415, 63422, 63430, 63437, 63444, 63451, 63458, 63465, 63472, 63479, 63486,  
63493, 63500, 63507, 63513, 63520, 63527, 63534, 63541, 63547, 63554, 63561, 63567, 63574, 63581, 63587,  
63594, 63600, 63607, 63613, 63620, 63626, 63633, 63639, 63646, 63652, 63658, 63665, 63671, 63677, 63684,  
63690, 63696, 63702, 63709, 63715, 63721, 63727, 63733, 63739, 63745, 63751, 63757, 63763, 63769, 63775,  
63781, 63787, 63793, 63799, 63805, 63811, 63817, 63822, 63828, 63834, 63840, 63845, 63851, 63857, 63862,  
63868, 63874, 63879, 63885, 63891, 63896, 63902, 63907, 63913, 63918, 63924, 63929, 63934, 63940, 63945,  
63951, 63956, 63961, 63967, 63972, 63977, 63982, 63988, 63993, 63998, 64003, 64009, 64014, 64019, 64024,  
64029, 64034, 64039, 64044, 64049, 64054, 64059, 64064, 64069, 64074, 64079, 64084, 64089, 64094, 64099,  
64104, 64108, 64113, 64118, 64123, 64128, 64132, 64137, 64142, 64147, 64151, 64156, 64161, 64165, 64170,  
64174, 64179, 64184, 64188, 64193, 64197, 64202, 64206, 64211, 64215, 64220, 64224, 64229, 64233, 64237,  
64242, 64246, 64251, 64255, 64259, 64263, 64268, 64272, 64276, 64281, 64285, 64289, 64293, 64297, 64302,  
64306, 64310, 64314, 64318, 64322, 64326, 64331, 64335, 64339, 64343, 64347, 64351, 64355, 64359, 64363,  
64367, 64371, 64375, 64378, 64382, 64386, 64390, 64394, 64398, 64402, 64406, 64409, 64413, 64417, 64421,  
64424, 64428, 64432, 64436, 64439, 64443, 64447, 64450, 64454, 64458, 64461, 64465, 64469, 64472, 64476,  
64479, 64483, 64487, 64490, 64494, 64497, 64501, 64504, 64508, 64511, 64515, 64518, 64521, 64525, 64528,  
64532, 64535, 64538, 64542, 64545, 64549, 64552, 64555, 64559, 64562, 64565, 64568, 64572, 64575, 64578,  
64581, 64585, 64588, 64591, 64594, 64597, 64601, 64604, 64607, 64610, 64613, 64616, 64619, 64622, 64626,  
64629, 64632, 64635, 64638, 64641, 64644, 64647, 64650, 64653, 64656, 64659, 64662, 64665, 64668, 64671,  
64673, 64676, 64679, 64682, 64685, 64688, 64691, 64694, 64696, 64699, 64702, 64705, 64708, 64711, 64713,  
64716, 64719, 64722, 64724, 64727, 64730, 64733, 64735, 64738, 64741, 64743, 64746, 64749, 64751, 64754,  
64757, 64759, 64762, 64764, 64767, 64770, 64772, 64775, 64777, 64780, 64783, 64785, 64788, 64790, 64793,  
64795, 64798, 64800, 64803, 64805, 64808, 64810, 64812, 64815, 64817, 64820, 64822, 64825, 64827, 64829,  
64832, 64834, 64837, 64839, 64841, 64844, 64846, 64848, 64851, 64853, 64855, 64857, 64860, 64862, 64864,  
64867, 64869, 64871, 64873, 64876, 64878, 64880, 64882, 64884, 64887, 64889, 64891, 64893, 64895, 64898,  
64900, 64902, 64904, 64906, 64908, 64910, 64912, 64915, 64917, 64919, 64921, 64923, 64925, 64927, 64929,  
64931, 64933, 64935, 64937, 64939, 64941, 64943, 64945, 64947, 64949, 64951, 64953, 64955, 64957, 64959,  
64961, 64963, 64965, 64967, 64969, 64971, 64973, 64974, 64976, 64978, 64980, 64982, 64984, 64986, 64988,  
64989, 64991, 64993, 64995, 64997, 64999, 65000, 65002, 65004, 65006, 65008, 65009, 65011, 65013, 65015,  
65016, 65018, 65020, 65022, 65023, 65025, 65027, 65029, 65030, 65032, 65034, 65035, 65037, 65039, 65040,  
65042, 65044, 65045, 65047, 65049, 65050, 65052, 65054, 65055, 65057, 65059, 65060, 65062, 65063, 65065,  
65067, 65068, 65070, 65071, 65073, 65074, 65076, 65077, 65079, 65081, 65082, 65084, 65085, 65087, 65088,  
65090, 65091, 65093, 65094, 65096, 65097, 65099, 65100, 65102, 65103, 65105, 65106, 65107, 65109, 65110,  
65112, 65113, 65115, 65116, 65117, 65119, 65120, 65122, 65123, 65124, 65126, 65127, 65129, 65130, 65131,  
65133, 65134, 65135, 65137, 65138, 65139, 65141, 65142, 65143, 65145, 65146, 65147, 65149, 65150, 65151,  
65153, 65154, 65155, 65156, 65158, 65159, 65160, 65162, 65163, 65164, 65165, 65167, 65168, 65169, 65170,  
65172, 65173, 65174, 65175, 65176, 65178, 65179, 65180, 65181, 65182, 65184, 65185, 65186, 65187, 65188,  
65190, 65191, 65192, 65193, 65194, 65195, 65196, 65198, 65199, 65200, 65201, 65202, 65203, 65204, 65206,  
65207, 65208, 65209, 65210, 65211, 65212, 65213, 65214, 65215, 65216, 65218, 65219, 65220, 65221, 65222,  
65223, 65224, 65225, 65226, 65227, 65228, 65229, 65230, 65231, 65232, 65233, 65234, 65235, 65236, 65237,  
65238, 65239, 65240, 65241, 65242, 65243, 65244, 65245, 65246, 65247, 65248, 65249, 65250, 65251, 65252,  
65253, 65254, 65255, 65256, 65257, 65258, 65259, 65260, 65261, 65261, 65262, 65263, 65264, 65265, 65266,  
65267, 65268, 65269, 65270, 65271, 65271, 65272, 65273, 65274, 65275, 65276, 65277, 65278, 65279, 65279,  
65280, 65281, 65282, 65283, 65284, 65285, 65285, 65286, 65287, 65288, 65289, 65290, 65290, 65291, 65292,  
65293, 65294, 65294, 65295 
};


uint16_t mapFreq(uint16_t input) {
  return pgm_read_word_near(freqTable + input);
}

const byte table_length = 32;
unsigned loop_table[table_length];

void setup()
{
	for (unsigned n = 0; n < table_length; n++) loop_table[n] = 2047;
	cli();							// disable global interrupts
	TCCR1A = 0;						// set entire TCCR1A register to 0
	TCCR1B = 0;						// same for TCCR1B
	TCCR1B =  0b00000001;			//scaling to a 8th of the CPU freq
	// enable Timer1 overflow interrupt:
	TIMSK1 |= (1 << TOIE1);			//interupt enable
	TCNT1 = 0;						//timer pre-loading
	sei();							//re-enable global interupts

	//set pin(s) to input and output
	pinMode(10, OUTPUT);
	SPI.begin();					// wake up the SPI bus. This is the fast way of controling the DAC
	SPI.setBitOrder(MSBFIRST);
	pinMode(trigger_pin, INPUT_PULLUP);
	pinMode(but_A_pin, INPUT_PULLUP);
	pinMode(but_B_pin, INPUT_PULLUP);
	trigger.pin = trigger_pin;
	but_A.pin = but_A_pin;
	but_B.pin = but_B_pin;
}

//send calculated value to the dac
void out( word value)
{
	byte data;						//spi deals in bytes (8 bits) chunks
	PORTB &= 0b11111011;			//enable writing
	data = highByte(value);			//this includes the 4 most significant bits
	data = 0b00001111 & data;		//the rest of the byte is settings for the dac: mask those
	data = 0b00110000 | data;		//set those setting bits (see DAC data sheet or trust me)
	SPI.transfer(data);				//actually send this out
	data = lowByte(value);			//the 8 least significant bits of our value
	SPI.transfer(data);				//send them
	PORTB |= 0b00000100;			//close connection, allowing the DAC to update}
}

//seeds the array with random numbers,
//then sets 'sustain' to on, meaning less damping while the trigger in high
void noteOn()
{
	for (byte n = 0; n < table_length; n++)
	{
		loop_table[n] = (rng() - 1) & 0b0000111111111111;
	}
	sustain = true;
}

//turn the damping off, which will make the note decay more quickly
void noteOff()
{
	sustain = false;
}

//actual work starts here

//the loop that reads the knobs, buttons and inputs
void loop()
{
	//read input 1, use the big lookkup table and set how often TIMER1 should
	//cause an interupt. This sets the samplerate and hence the pitch
	timer1_pre_load = mapFreq( analogRead(mod_a_pin) ); //10 bits
	//read input 2 and reduce it to 4 bits. We calculate in 16 bits and the DAC
	//is 12 bit, so we have 4 bits of headroom to play around with for our filter
	mix_A = max(1, analogRead(mod_b_pin) >> 6);
	mix_B = 16 - mix_A;
	//read and debounce buttons and the trigger
	if( updateButton( &trigger))
	{
		if( trigger.state)
		{
			noteOn();
		}
		else noteOff();
		digitalWrite(led_a, trigger.state);
	}
	if( updateButton( &but_A))
	{
		if( but_A.state)
		{
			noteOn();
		}
		else noteOff();
	}
	if( updateButton( &but_B))
	{
		if(but_B.state)
		{
			bowed = !bowed;
			digitalWrite(led_b, bowed);
		}
	}
}

//this gets called once it's time to generate a new sample
ISR(TIMER1_OVF_vect)
{
	//start be pre-loading the timer so the interrupt will trigger on time
	//for the next sample
	TCNT1 = timer1_pre_load;
	//increase the counter that loop through the table and make it wrap
	phase++;
	phase &= 0b00011111;
	output = loop_table[phase];
	//after the note-off we decay more quickly
	//we do that by mixing 15 parts of signal with 1 part of 0 (2047 is the 
	//middle of the DAC's range) and dividing by 16
	if (!sustain)
	{
		output = ((output * 15) + 2047) >> 4;
	}
	//add noise during 'bowed' mode, same trick as above
	else if (bowed)
	{
		output = (output * 15) + ((rng() - 1) & 0b0000111111111111);
		output = output >> 4;
	}
	//our simple averaging lowpass filter
	output = ((output * mix_A) + (last_out * mix_B)) >> 4;
	//store the output value in 'lastout' so we can use it next time 'round.
	last_out = output;
	//inverting feedback loop
	loop_table[phase] = 4095 - output;
	//and we're done and ready to present the listener with a new value
	out( output  );
}

