/*
	"Kitchen"; a Tool for making percussive noises
	Copyright (C) 2017  Kassen Oud


	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2 of the License, or (at your option)
	any later version.

	This program is distributed in the hope that it will be useful, but WITHOUT
	ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
	FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
	more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc., 51
	Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


	Configuration;
		It's recommended to keep the jumper on the DAC submodule when running
		this code.  This way your sound should stay in the -5V to +5V range.
		please note that as this code uses close to all available memory it
		will take longer than most codes to upload.


	Manual;
	This code provides you with 13 short, mostly noisy, percussive samples.
	Of those 2 will be loaded at any one time, let's call those "SampleA" and
	"SampleB", for convenience. You can pick the samples you want to use
	independently and you can cross-fade between your choices. You can also
	re-pitch the playback, ranging from half speed to double speed.

	Tr/G; Start playback of both selected samples.

	knob & CV 1; pitch, with the 12 O'Clock postition being the pitch at which
			the sound was recorded. This ranges from 50% to 200% playback
			speed.  Note that this is not a "propper" V/Oct tuning and instead
			focusses on the most usable range.
	knob 2 & CV 2; x-fade between SampleA and SampleB
	button A; select SampleA (left side of the x-fade)
	button B; select SampleB (right side of the x-fade)
			Note that these buttons cycle through the available samples, but
			will never load the same sound on both slots.

	Rationale;
		The history of this code actually goes back for years. I was in a
		debate on sample libraries for samplers. I found (and find) those a
		really boring use for samplers and tried to plead that it'd be much
		more exciting to sample the various things in your kitchen. I stand by
		that, but never actually got round to it, so here it is. Let's note the
		amusing irony that the result is basically a sample library that you
		downloaded (but of course you could replace all of these with your own,
		and I hope you do!).

		Most of these were recorded by striking various kitchen objects with a
		metal chopstick, lovingly recorded using a Rode NT1-A microphone
		generously lended by Robert Beenen, straight into my my Native
		Instruments Komplete Audio6, with no further processing, then brutally
		converted to 8 bit 16KHz. After that an entirely arbitrary selection
		was made because memory space is limited on the Arduino. This led to
		most longer samples getting discarded. Fortunately it turned out that
		the longer ones were also mostly the more tonal ones and that the
		samples with a clearer pitch were not that versatile in practical use.
		I strove for a mix of sounds with diverse tonal characteristics, but it
		turned out that it is also very interesting to have some quite similar
		samples in there as those turn out to be more useful to crossfade
		between.

 */
 
#include "SPI.h" // spi library
#include <avr/interrupt.h> //library for using interupts
#include <avr/pgmspace.h>	//to store the samples in program memory
#include "kas-button.h" //button debouncing

// files holding the sampled data in 8 bit as arrays
#include "coffee1.h"
#include "coffee2.h"
#include "cups1.h"
#include "cups2.h"
#include "percolator2.h"
#include "percolator.h"
#include "pol1.h"
#include "pol2.h"
#include "spatel1.h"
#include "spatel2.h"
#include "thermos1.h"
#include "beer1.h"
#include "beer2.h"

static byte mod_a_pin = A1;
static byte mod_b_pin = A2;
static byte trigger_pin = 4;
static byte but_A_pin = 2;
static byte but_B_pin = 9;
static byte led_a = 3;
//static byte led_b = 5;

//if you add or remove samples you should change this number
static byte num_samps = 13;

//we need to keep track of how long each samples is
static uint16_t samp_lengths[] = {
	coffee1_8_wav_len,
	coffee2_8_wav_len,
	cups1_8_wav_len,
	cups2_8_wav_len,
	percolator_8_wav_len,
	percolator2_8_wav_len,
	pol1_8_wav_len,
	pol2_8_wav_len,
	spatel1_8_wav_len,
	spatel2_8_wav_len,
	thermos1_8_wav_len,
	beer1_8_wav_len,
	beer2_8_wav_len
	};


byte sampA = 0;
byte sampB = 1;
bool playing = false;
uint16_t max_length = samp_lengths[ (samp_lengths[sampA] > samp_lengths[sampB])? sampA: sampB ];

static uint16_t zero = pow(2, 7);

int timer1_pre_load = 1;
unsigned phase = 0;
unsigned max_phase = 0;

Button trigger;
Button but_A;
Button but_B;

uint16_t output = zero;

byte mix_A = 15;
byte mix_B = 0;

unsigned char readSample(uint16_t input, byte samp)
{
	if (input >= samp_lengths[samp]) return 127;
	//please note that it is important that the samples here are in the same order
	//as the list of the length of these samples
	else switch (samp)
	{
		case 0:
			return pgm_read_byte(coffee1_data + input );
		case 1:
			return pgm_read_byte(coffee2_data + input );
		case 2:
			return pgm_read_byte(cups1_data + input );
		case 3:
			return pgm_read_byte(cups2_data + input );
		case 4:
			return pgm_read_byte(percolator_data + input );
		case 5:
			return pgm_read_byte(percolator2_data + input );
		case 6:
			return pgm_read_byte(pol1_data + input );
		case 7:
			return pgm_read_byte(pol2_data + input );
		case 8:
			return pgm_read_byte(spatel1_data + input );
		case 9:
			return pgm_read_byte(spatel2_data + input );
		case 10:
			return pgm_read_byte(thermos1_data + input );
		case 11:
			return pgm_read_byte(beer1_data + input );
		case 12:
			return pgm_read_byte(beer2_data + input );
		//this last case should never happen, but better safe than sorry
		default:
			return 127;
	}
}

//exponential table for pre-loading timer1
//this takes care of the playback speed tuning
const uint16_t freqTable[] PROGMEM = {
65285, 65285, 65286, 65286, 65286, 65287, 65287, 65287, 65288, 65288, 65288, 65289, 65289, 65289, 65290,
65290, 65290, 65291, 65291, 65291, 65292, 65292, 65292, 65293, 65293, 65293, 65294, 65294, 65294, 65295,
65295, 65295, 65296, 65296, 65296, 65297, 65297, 65297, 65298, 65298, 65298, 65298, 65299, 65299, 65299,
65300, 65300, 65300, 65301, 65301, 65301, 65302, 65302, 65302, 65303, 65303, 65303, 65304, 65304, 65304,
65305, 65305, 65305, 65305, 65306, 65306, 65306, 65307, 65307, 65307, 65308, 65308, 65308, 65309, 65309,
65309, 65309, 65310, 65310, 65310, 65311, 65311, 65311, 65312, 65312, 65312, 65312, 65313, 65313, 65313,
65314, 65314, 65314, 65315, 65315, 65315, 65315, 65316, 65316, 65316, 65317, 65317, 65317, 65318, 65318,
65318, 65318, 65319, 65319, 65319, 65320, 65320, 65320, 65320, 65321, 65321, 65321, 65322, 65322, 65322,
65322, 65323, 65323, 65323, 65324, 65324, 65324, 65324, 65325, 65325, 65325, 65326, 65326, 65326, 65326,
65327, 65327, 65327, 65328, 65328, 65328, 65328, 65329, 65329, 65329, 65330, 65330, 65330, 65330, 65331,
65331, 65331, 65331, 65332, 65332, 65332, 65333, 65333, 65333, 65333, 65334, 65334, 65334, 65335, 65335,
65335, 65335, 65336, 65336, 65336, 65336, 65337, 65337, 65337, 65337, 65338, 65338, 65338, 65339, 65339,
65339, 65339, 65340, 65340, 65340, 65340, 65341, 65341, 65341, 65341, 65342, 65342, 65342, 65342, 65343,
65343, 65343, 65344, 65344, 65344, 65344, 65345, 65345, 65345, 65345, 65346, 65346, 65346, 65346, 65347,
65347, 65347, 65347, 65348, 65348, 65348, 65348, 65349, 65349, 65349, 65349, 65350, 65350, 65350, 65350,
65351, 65351, 65351, 65351, 65352, 65352, 65352, 65352, 65353, 65353, 65353, 65353, 65354, 65354, 65354,
65354, 65355, 65355, 65355, 65355, 65356, 65356, 65356, 65356, 65357, 65357, 65357, 65357, 65358, 65358,
65358, 65358, 65358, 65359, 65359, 65359, 65359, 65360, 65360, 65360, 65360, 65361, 65361, 65361, 65361,
65362, 65362, 65362, 65362, 65362, 65363, 65363, 65363, 65363, 65364, 65364, 65364, 65364, 65365, 65365,
65365, 65365, 65365, 65366, 65366, 65366, 65366, 65367, 65367, 65367, 65367, 65368, 65368, 65368, 65368,
65368, 65369, 65369, 65369, 65369, 65370, 65370, 65370, 65370, 65370, 65371, 65371, 65371, 65371, 65372,
65372, 65372, 65372, 65372, 65373, 65373, 65373, 65373, 65374, 65374, 65374, 65374, 65374, 65375, 65375,
65375, 65375, 65376, 65376, 65376, 65376, 65376, 65377, 65377, 65377, 65377, 65377, 65378, 65378, 65378,
65378, 65379, 65379, 65379, 65379, 65379, 65380, 65380, 65380, 65380, 65380, 65381, 65381, 65381, 65381,
65381, 65382, 65382, 65382, 65382, 65382, 65383, 65383, 65383, 65383, 65384, 65384, 65384, 65384, 65384,
65385, 65385, 65385, 65385, 65385, 65386, 65386, 65386, 65386, 65386, 65387, 65387, 65387, 65387, 65387,
65388, 65388, 65388, 65388, 65388, 65389, 65389, 65389, 65389, 65389, 65390, 65390, 65390, 65390, 65390,
65391, 65391, 65391, 65391, 65391, 65391, 65392, 65392, 65392, 65392, 65392, 65393, 65393, 65393, 65393,
65393, 65394, 65394, 65394, 65394, 65394, 65395, 65395, 65395, 65395, 65395, 65396, 65396, 65396, 65396,
65396, 65396, 65397, 65397, 65397, 65397, 65397, 65398, 65398, 65398, 65398, 65398, 65399, 65399, 65399,
65399, 65399, 65399, 65400, 65400, 65400, 65400, 65400, 65401, 65401, 65401, 65401, 65401, 65401, 65402,
65402, 65402, 65402, 65402, 65403, 65403, 65403, 65403, 65403, 65403, 65404, 65404, 65404, 65404, 65404,
65404, 65405, 65405, 65405, 65405, 65405, 65406, 65406, 65406, 65406, 65406, 65406, 65407, 65407, 65407,
65407, 65407, 65407, 65408, 65408, 65408, 65408, 65408, 65408, 65409, 65409, 65409, 65409, 65409, 65409,
65410, 65410, 65410, 65410, 65410, 65411, 65411, 65411, 65411, 65411, 65411, 65412, 65412, 65412, 65412,
65412, 65412, 65413, 65413, 65413, 65413, 65413, 65413, 65414, 65414, 65414, 65414, 65414, 65414, 65414,
65415, 65415, 65415, 65415, 65415, 65415, 65416, 65416, 65416, 65416, 65416, 65416, 65417, 65417, 65417,
65417, 65417, 65417, 65418, 65418, 65418, 65418, 65418, 65418, 65418, 65419, 65419, 65419, 65419, 65419,
65419, 65420, 65420, 65420, 65420, 65420, 65420, 65421, 65421, 65421, 65421, 65421, 65421, 65421, 65422,
65422, 65422, 65422, 65422, 65422, 65423, 65423, 65423, 65423, 65423, 65423, 65423, 65424, 65424, 65424,
65424, 65424, 65424, 65424, 65425, 65425, 65425, 65425, 65425, 65425, 65426, 65426, 65426, 65426, 65426,
65426, 65426, 65427, 65427, 65427, 65427, 65427, 65427, 65427, 65428, 65428, 65428, 65428, 65428, 65428,
65428, 65429, 65429, 65429, 65429, 65429, 65429, 65429, 65430, 65430, 65430, 65430, 65430, 65430, 65430,
65431, 65431, 65431, 65431, 65431, 65431, 65431, 65432, 65432, 65432, 65432, 65432, 65432, 65432, 65433,
65433, 65433, 65433, 65433, 65433, 65433, 65434, 65434, 65434, 65434, 65434, 65434, 65434, 65434, 65435,
65435, 65435, 65435, 65435, 65435, 65435, 65436, 65436, 65436, 65436, 65436, 65436, 65436, 65437, 65437,
65437, 65437, 65437, 65437, 65437, 65437, 65438, 65438, 65438, 65438, 65438, 65438, 65438, 65438, 65439,
65439, 65439, 65439, 65439, 65439, 65439, 65440, 65440, 65440, 65440, 65440, 65440, 65440, 65440, 65441,
65441, 65441, 65441, 65441, 65441, 65441, 65441, 65442, 65442, 65442, 65442, 65442, 65442, 65442, 65442,
65443, 65443, 65443, 65443, 65443, 65443, 65443, 65443, 65444, 65444, 65444, 65444, 65444, 65444, 65444,
65444, 65445, 65445, 65445, 65445, 65445, 65445, 65445, 65445, 65446, 65446, 65446, 65446, 65446, 65446,
65446, 65446, 65446, 65447, 65447, 65447, 65447, 65447, 65447, 65447, 65447, 65448, 65448, 65448, 65448,
65448, 65448, 65448, 65448, 65449, 65449, 65449, 65449, 65449, 65449, 65449, 65449, 65449, 65450, 65450,
65450, 65450, 65450, 65450, 65450, 65450, 65450, 65451, 65451, 65451, 65451, 65451, 65451, 65451, 65451,
65451, 65452, 65452, 65452, 65452, 65452, 65452, 65452, 65452, 65453, 65453, 65453, 65453, 65453, 65453,
65453, 65453, 65453, 65454, 65454, 65454, 65454, 65454, 65454, 65454, 65454, 65454, 65454, 65455, 65455,
65455, 65455, 65455, 65455, 65455, 65455, 65455, 65456, 65456, 65456, 65456, 65456, 65456, 65456, 65456,
65456, 65457, 65457, 65457, 65457, 65457, 65457, 65457, 65457, 65457, 65457, 65458, 65458, 65458, 65458,
65458, 65458, 65458, 65458, 65458, 65459, 65459, 65459, 65459, 65459, 65459, 65459, 65459, 65459, 65459,
65460, 65460, 65460, 65460, 65460, 65460, 65460, 65460, 65460, 65460, 65461, 65461, 65461, 65461, 65461,
65461, 65461, 65461, 65461, 65461, 65462, 65462, 65462, 65462, 65462, 65462, 65462, 65462, 65462, 65462,
65463, 65463, 65463, 65463, 65463, 65463, 65463, 65463, 65463, 65463, 65464, 65464, 65464, 65464, 65464,
65464, 65464, 65464, 65464, 65464, 65464, 65465, 65465, 65465, 65465, 65465, 65465, 65465, 65465, 65465,
65465, 65466, 65466, 65466, 65466, 65466, 65466, 65466, 65466, 65466, 65466, 65466, 65467, 65467, 65467,
65467, 65467, 65467, 65467, 65467, 65467, 65467, 65467, 65468, 65468, 65468, 65468, 65468, 65468, 65468,
65468, 65468, 65468, 65468, 65469, 65469, 65469, 65469, 65469, 65469, 65469, 65469, 65469, 65469, 65469,
65470, 65470, 65470, 65470, 65470, 65470, 65470, 65470, 65470, 65470, 65470, 65471, 65471, 65471, 65471,
65471, 65471, 65471, 65471, 65471, 65471, 65471, 65471, 65472, 65472, 65472, 65472, 65472, 65472, 65472,
65472, 65472, 65472, 65472
};


uint16_t mapFreq(uint16_t input) {
  return pgm_read_word_near(freqTable + input);
}


void setup()
{
	cli();							// disable global interrupts
	TCCR1A = 0;						// set entire TCCR1A register to 0
	TCCR1B = 0;						// same for TCCR1B
	TCCR1B =  0b00000010;			//scaling to a 8th of the CPU freq
	// enable Timer1 overflow interrupt:
	TIMSK1 |= (1 << TOIE1);			//interupt enable
	TCNT1 = 0;						//timer pre-loading
	sei();							//re-enable global interupts

	//set pin(s) to input and output
	pinMode(10, OUTPUT);
	SPI.begin();					// wake up the SPI bus. This is the fast way of controling the DAC
	SPI.setBitOrder(MSBFIRST);
	pinMode(trigger_pin, INPUT_PULLUP);
	pinMode(but_A_pin, INPUT_PULLUP);
	pinMode(but_B_pin, INPUT_PULLUP);
	trigger.pin = trigger_pin;
	but_A.pin = but_A_pin;
	but_B.pin = but_B_pin;
}

//send calculated value to the dac
void out( word value)
{
	byte data;						//spi deals in bytes (8 bits) chunks
	PORTB &= 0b11111011;			//enable writing
	data = highByte(value);			//this includes the 4 most signifficant bits
	data = 0b00001111 & data;		//the rest of the byte is settings for the dac: mask those
	data = 0b00110000 | data;		//set those setting bits (see DAC data sheet or trust me)
	SPI.transfer(data);				//actually send this out
	data = lowByte(value);			//the 8 least signifficant bits of our value
	SPI.transfer(data);				//send them
	PORTB |= 0b00000100;			//close connection, allowing the DAC to update}
}


void loop()
{
	timer1_pre_load = mapFreq( analogRead(mod_a_pin) ); //10 bits
	mix_B = min (255, analogRead(mod_b_pin) >> 2);
	mix_A = 255 - mix_B;
	if( updateButton( &trigger))
	{
		if( trigger.state)
		{
			phase = 0;
			playing = true;
		}
		digitalWrite(led_a, trigger.state);
	}
	if( updateButton( &but_A))
	{
		if(but_A.state)
		{
			sampA = (sampA + 1) % num_samps;
			if (sampA == sampB) sampA = (sampA + 1) % num_samps;
			max_length = samp_lengths[ (samp_lengths[sampA] > samp_lengths[sampB])? sampA: sampB ];
		}
	}
	if( updateButton( &but_B))
	{
		if(but_B.state)
		{
			sampB = (sampB + 1) % num_samps;
			if (sampA == sampB) sampB = (sampB + 1) % num_samps;
			max_length = samp_lengths[ (samp_lengths[sampA] > samp_lengths[sampB])? sampA: sampB ];
		}
	}
}


//actual sound output gets calculated here.
//this function gets triggered as timer1 overflows.
//the rate at which this happens depends on the "pre-loading" which comes from the big table above
//and the number gets selected by CV1
ISR(TIMER1_OVF_vect)
{
	TCNT1 = timer1_pre_load;
	output = 0;
	if (playing)
	{
		phase++;
		if (phase >= max_length)
		{
			playing = false;
			//2048 is halfway of the range of the DAC, so it's what we'd consider silence
			output = 2048;
		}
		else
		{
			output += (readSample(phase, sampA) * mix_A) >> 4;
			output += (readSample(phase, sampB) * mix_B) >> 4;
		}
	}
	else
	{
		output = 2048;
	}
	out( output );
}

