/*
	"Bag"; a funky kind of envelope.
	
		Copyright (C) 2017  Kassen Oud
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


		Configuration;
	It is recommended to remove the jumper from the DAC module of your Tool when
	using this; the ouput is intended to be unipolar. When configured like this
	a single trigger should make the output range from 0 to (about) +5V.
	Repeatedly triggering the module may make it go up to as high as +10V.
			
		MANUAL;
	An envelope generator that was quite heavily influenced by Robin Whittle's
	writing on the TB303's envelope, found at;
	http://www.firstpr.com.au/rwi/dfish/303-unique.html
	In particular the phrase "There are a number of things that only the TB-303 
	can do - which cannot be done with a modular synth using standard modules.".
	Now times have changed since 1999, but I do agree with Mr Whittle that the
	TB303's envelope's behaviour is more interesting than the debate on the 
	exact nature of the machine's filter. And well; if modular synths can't do 
	this, they ought to be able to. Let's give it a shot.

	Knob 1 & Mod input 1; 
		Set the envelope's attack time.
	Knob 2 & Mod input 2; 
		Set the envelope's release time.
	TR/G input & button A;
		trigger the envelope. Note that re-triggering it before it has had the
		time to return to 0 will add to the current output value. This means
		that a series of successive pulses will yield you an increasing ouput.
	Button B;
		Toggles wheter we'll "sustain", to some degree, while the TR/G input is
		high. Without sustain the module will go from the 'attack' phase
		straight to the release. With sustain the output will decay relatively
		slowly (at laf rate) until the TR/G input goes low, after which it'll
		decrease more quickly.
	Led A;
		Indicates the value on the TR/G input.
	Led B;
		Indicates whether sustain is toggled on or off.
		

 */
 
#include "SPI.h" // spi library
#include <avr/interrupt.h> //library for using interupts
#include <avr/pgmspace.h>
#include "kas-button.h" //button debouncing

static byte mod_a_pin = A1;
static byte mod_b_pin = A2;
static byte trigger_pin = 4;
static byte but_A_pin = 2;
static byte but_B_pin = 9;
static byte led_a = 3;
static byte led_b = 5;

static unsigned long max_u_long = 4294967295;
static unsigned long max_u_int = 65535;

uint32_t value = 0;
uint32_t damp = 4000;
uint32_t impulse = 3240;
unsigned gravity = 1;
bool triggered = false;
bool sustain = false;



word mod_a_val = 0;
word mod_b_val = 0;
Button trigger;
Button but_A;
Button but_B;

const uint16_t imp_table[] PROGMEM = {
 53800,  52758,  51736,  50734,  49751,  48788,  47843,  46916,  46008,  45117,  44243,  43386,  42545,
 41721,  40913,  40121,  39344,  38582,  37835,  37102,  36383,  35679,  34988,  34310,  33645,  32994,
 32355,  31728,  31113,  30511,  29920,  29340,  28772,  28215,  27668,  27133,  26607,  26092,  25586,
 25091,  24605,  24128,  23661,  23203,  22753,  22313,  21880,  21457,  21041,  20634,  20234,  19842,
 19458,  19081,  18711,  18349,  17993,  17645,  17303,  16968,  16639,  16317,  16001,  15691,  15387,
 15089,  14797,  14510,  14229,  13954,  13684,  13418,  13159,  12904,  12654,  12409,  12168,  11933,
 11702,  11475,  11253,  11035,  10821,  10611,  10406,  10204,  10007,   9813,   9623,   9436,   9254,
  9074,   8899,   8726,   8557,   8392,   8229,   8070,   7913,   7760,   7610,   7462,   7318,   7176,
  7037,   6901,   6767,   6636,   6508,   6382,   6258,   6137,   6018,   5901,   5787,   5675,   5565,
  5457,   5352,   5248,   5146,   5047,   4949,   4853,   4759,   4667,   4576,   4488,   4401,   4316,
  4232,   4150,   4070,   3991,   3914,   3838,   3763,   3691,   3619,   3549,   3480,   3413,   3347,
  3282,   3218,   3156,   3095,   3035,   2976,   2919,   2862,   2807,   2752,   2699,   2647,   2595,
  2545,   2496,   2447,   2400,   2354,   2308,   2263,   2219,   2176,   2134,   2093,   2052,   2013,
  1974,   1935,   1898,   1861,   1825,   1790,   1755,   1721,   1688,   1655,   1623,   1592,   1561,
  1531,   1501,   1472,   1443,   1415,   1388,   1361,   1335,   1309,   1284,   1259,   1234,   1210,
  1187,   1164,   1141,   1119,   1098,   1076,   1056,   1035,   1015,    995,    976,    957,    939,
   920,    903,    885,    868,    851,    835,    819,    803,    787,    772,    757,    742,    728,
   714,    700,    686,    673,    660,    647,    635,    622,    610,    599,    587,    576,    565,
   554,    543,    532,    522,    512,    502,    492,    483,    473,    464,    455,    446,    438,
   429,    421,    413,    405,    397,    389,    382,    374,    367
};


uint16_t mapImp(uint16_t input) {
  return pgm_read_word_near(imp_table + input);
}

const uint16_t damp_table[] PROGMEM = {
 53800,  52578,  51384,  50216,  49076,  47961,  46871,  45807,  44766,  43749,  42755,  41784,  40835,
 39907,  39001,  38115,  37249,  36403,  35576,  34768,  33978,  33206,  32452,  31715,  30994,  30290,
 29602,  28930,  28272,  27630,  27003,  26389,  25790,  25204,  24631,  24072,  23525,  22991,  22468,
 21958,  21459,  20972,  20495,  20030,  19575,  19130,  18695,  18271,  17856,  17450,  17054,  16666,
 16288,  15918,  15556,  15203,  14857,  14520,  14190,  13868,  13553,  13245,  12944,  12650,  12363,
 12082,  11807,  11539,  11277,  11021,  10770,  10526,  10287,  10053,   9825,   9601,   9383,   9170,
  8962,   8758,   8559,   8365,   8175,   7989,   7808,   7630,   7457,   7288,   7122,   6960,   6802,
  6648,   6497,   6349,   6205,   6064,   5926,   5792,   5660,   5531,   5406,   5283,   5163,   5046,
  4931,   4819,   4710,   4603,   4498,   4396,   4296,   4198,   4103,   4010,   3919,   3830,   3743,
  3658,   3575,   3493,   3414,   3337,   3261,   3187,   3114,   3044,   2974,   2907,   2841,   2776,
  2713,   2652,   2591,   2532,   2475,   2419,   2364,   2310,   2258,   2206,   2156,   2107,   2059,
  2013,   1967,   1922,   1878,   1836,   1794,   1753,   1714,   1675,   1637,   1599,   1563,   1528,
  1493,   1459,   1426,   1393,   1362,   1331,   1301,   1271,   1242,   1214,   1186,   1159,   1133,
  1107,   1082,   1058,   1034,   1010,    987,    965,    943,    921,    900,    880,    860,    840,
   821,    803,    785,    767,    749,    732,    716,    699,    683,    668,    653,    638,    623,
   609,    595,    582,    569,    556,    543,    531,    519,    507,    495,    484,    473,    462,
   452,    442,    432,    422,    412,    403,    394,    385,    376,    368,    359,    351,    343,
   335,    328,    320,    313,    306,    299,    292,    285,    279,    273,    266,    260,    254,
   249,    243,    238,    232,    227,    222,    217,    212,    207,    202,    198,    193,    189,
   184,    180,    176,    172,    168,    164,    161,    157,    153
};


uint16_t mapDamp(uint16_t input) {
  return pgm_read_word_near(damp_table + input);
}
const uint16_t grav_table[] PROGMEM = {
   500,    496,    491,    487,    482,    478,    474,    469,    465,    461,    457,    453,    449,
   445,    441,    437,    433,    429,    425,    421,    418,    414,    410,    407,    403,    399,
   396,    392,    389,    385,    382,    378,    375,    372,    368,    365,    362,    358,    355,
   352,    349,    346,    343,    340,    337,    334,    331,    328,    325,    322,    319,    316,
   313,    310,    308,    305,    302,    299,    297,    294,    291,    289,    286,    284,    281,
   279,    276,    274,    271,    269,    266,    264,    262,    259,    257,    255,    252,    250,
   248,    246,    243,    241,    239,    237,    235,    233,    231,    229,    227,    225,    223,
   221,    219,    217,    215,    213,    211,    209,    207,    205,    203,    202,    200,    198,
   196,    194,    193,    191,    189,    188,    186,    184,    183,    181,    179,    178,    176,
   175,    173,    171,    170,    168,    167,    165,    164,    162,    161,    160,    158,    157,
   155,    154,    153,    151,    150,    148,    147,    146,    145,    143,    142,    141,    139,
   138,    137,    136,    134,    133,    132,    131,    130,    129,    127,    126,    125,    124,
   123,    122,    121,    120,    119,    118,    116,    115,    114,    113,    112,    111,    110,
   109,    108,    107,    106,    105,    105,    104,    103,    102,    101,     100,     99,     98,
    97,     96,     96,     95,     94,     93,     92,     91,     91,     90,     89,     88,     87,
    87,     86,     85,     84,     83,     83,     82,     81,     81,     80,     79,     78,     78,
    77,     76,     76,     75,     74,     74,     73,     72,     72,     71,     70,     70,     69,
    68,     68,     67,     67,     66,     65,     65,     64,     64,     63,     63,     62,     61,
    61,     60,     60,     59,     59,     58,     58,     57,     57,     56,     56,     55,     55,
    54,     54,     53,     53,     52,     52,     51,     51,     50,
};


uint16_t mapGrav(uint16_t input) {
  return pgm_read_word_near(grav_table + input);
}


void setup()
{
	cli();							// disable global interrupts
	TCCR2A = 0;						// set entire TCCR1A register to 0
	TCCR2B = 0;						// same for TCCR1B
	TCCR2B =  0b00000011;			//scaling to a 256th of the CPU freq = 31250 Hz
	// enable Timer1 overflow interrupt:
	TIMSK2 |= (1 << TOIE1);			//interupt enable
	TCNT2 = 0;						//timer pre-loading
	sei();							//re-enable global interupts

	//set pin(s) to input and output
	pinMode(10, OUTPUT);
	SPI.begin();					// wake up the SPI bus. This is the fast way of controling the DAC
	SPI.setBitOrder(MSBFIRST);
	//deal with pinmodes and link those to debouncing lib
	pinMode(trigger_pin, INPUT_PULLUP);
	pinMode(but_A_pin, INPUT_PULLUP);
	pinMode(but_B_pin, INPUT_PULLUP);
	trigger.pin = trigger_pin;
	but_A.pin = but_A_pin;
	but_B.pin = but_B_pin;
}

//send calculated value to the dac
void out( word value)
{
	byte data;						//spi deals in bytes (8 bits) chunks
	PORTB &= 0b11111011;			//enable writing
	data = highByte(value);			//this includes the 4 most signifficant bits
	data = 0b00001111 & data;		//the rest of the byte is settings for the dac: mask those
	data = 0b00110000 | data;		//set those setting bits (see DAC data sheet or trust me)
	SPI.transfer(data);				//actually send this out
	data = lowByte(value);			//the 8 least signifficant bits of our value
	SPI.transfer(data);				//send them
	PORTB |= 0b00000100;			//close connection, allowing the DAC to update
}

void onTrigger()
{
	impulse = mapImp(mod_a_val);
	damp = mapDamp(mod_a_val);
	triggered = true;
}

void onRelease()
{
	triggered = false;
}

void loop()
{
	mod_a_val = analogRead(mod_a_pin) >> 2;
	mod_b_val = analogRead(mod_b_pin) >> 2;
	gravity = mapGrav(mod_b_val);
	if( updateButton( &trigger))
	{
		if (trigger.state)
		{
			onTrigger();
		}
		else
		{
			onRelease();
		}
		digitalWrite(led_a, trigger.state);
	}
	if( updateButton( &but_A))
	{
		if(but_A.state)
		{
			onTrigger();
		}
	}
	if( updateButton( &but_B))
	{
		if(but_B.state)
		{
			sustain = !sustain;
			digitalWrite(led_b, sustain);
		}
	}
}


ISR(TIMER2_OVF_vect)
{
	if (impulse > 0)
	{
		value = min(131072, value + impulse);
		impulse *= (65536 - damp);
		impulse = impulse >> 16;
	}
	else
	{
		if (sustain && triggered)
		{
			value *= (32768 - (gravity >> 1));
		}
		else
		{
			value *= (32768 - gravity);
		}
		value = value >> 15;
	}
	out( min(4095, value >> 5) );
}

